@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
  */
 
 #include <gtk/gtk.h>
@@ -37,12 +37,10 @@ struct _GucharmapChartablePrivate {
   /* scrollable implementation */
   GtkAdjustment *vadjustment;
   gulong vadjustment_changed_handler_id;
-#if GTK_CHECK_VERSION (2, 91, 2)
   /* GtkScrollable impl */
   GtkAdjustment *hadjustment; /* unused */
   guint hscroll_policy : 1; /* unused */
   guint vscroll_policy : 1;
-#endif
 
   /* Font */
   PangoFontDescription *font_desc;
