@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
  */
 
 #include <config.h>
@@ -149,8 +149,6 @@ matches (GucharmapSearchDialog *search_dialog,
 
   if (annotations)
     {
-
-#if ENABLE_UNIHAN
       haystack = gucharmap_get_unicode_kDefinition (wc);
       if (haystack)
 	{
@@ -163,7 +161,6 @@ matches (GucharmapSearchDialog *search_dialog,
 
       if (matched)
 	return TRUE;
-#endif
 
       haystack_arr = gucharmap_get_nameslist_equals (wc);
       if (haystack_arr)
@@ -555,11 +552,7 @@ information_dialog (GucharmapSearchDialog *search_dialog,
   GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
   GtkWidget *dialog;
 
-#if GTK_CHECK_VERSION (2,18,0)
   dialog = gtk_message_dialog_new (gtk_widget_get_visible (GTK_WIDGET (search_dialog)) ?
-#else
-  dialog = gtk_message_dialog_new (GTK_WIDGET_VISIBLE (search_dialog) ?
-#endif
                                      GTK_WINDOW (search_dialog) :
                                      GTK_WINDOW (priv->guw),
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -591,7 +584,8 @@ search_completed (GucharmapSearchDialog *search_dialog)
       gtk_widget_set_sensitive (priv->next_button, FALSE);
     }
 
-  gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (search_dialog)), NULL);
+  if (gtk_widget_get_realized (GTK_WIDGET (search_dialog)))
+      gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (search_dialog)), NULL);
 }
 
 static gboolean
@@ -620,9 +614,11 @@ _gucharmap_search_dialog_fire_search (GucharmapSearchDialog *search_dialog,
   if (priv->search_state && priv->search_state->searching) /* Already searching */
     return;
 
-  cursor = gdk_cursor_new_for_display (gtk_widget_get_display (GTK_WIDGET (search_dialog)), GDK_WATCH);
-  gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (search_dialog)), cursor);
-  gdk_cursor_unref (cursor);
+  if (gtk_widget_get_realized (GTK_WIDGET (search_dialog))) {
+    cursor = gdk_cursor_new_for_display (gtk_widget_get_display (GTK_WIDGET (search_dialog)), GDK_WATCH);
+    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (search_dialog)), cursor);
+    g_object_unref (cursor);
+  }
 
   list = gucharmap_charmap_get_book_codepoint_list (priv->guw->charmap);
   if (!list)
@@ -667,7 +663,7 @@ gucharmap_search_dialog_start_search (GucharmapSearchDialog *search_dialog,
 {
   GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
 
-  if (priv->search_state != NULL && !_entry_is_empty (GTK_ENTRY (priv->entry)))
+  if (!_entry_is_empty (GTK_ENTRY (priv->entry)))
     _gucharmap_search_dialog_fire_search (search_dialog, direction);
   else
     gtk_window_present (GTK_WINDOW (search_dialog));
@@ -728,16 +724,13 @@ static void
 gucharmap_search_dialog_init (GucharmapSearchDialog *search_dialog)
 {
   GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
-  GtkWidget *hbox, *label, *content_area;
+  GtkWidget *grid, *label, *content_area;
 
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (search_dialog));
 
   /* follow hig guidelines */
   gtk_window_set_title (GTK_WINDOW (search_dialog), _("Find"));
   gtk_container_set_border_width (GTK_CONTAINER (search_dialog), 6);
-#if !GTK_CHECK_VERSION (2, 90, 7)
-  gtk_dialog_set_has_separator (GTK_DIALOG (search_dialog), FALSE);
-#endif
   gtk_window_set_destroy_with_parent (GTK_WINDOW (search_dialog), TRUE);
   gtk_box_set_spacing (GTK_BOX (content_area), 12);
   gtk_window_set_resizable (GTK_WINDOW (search_dialog), FALSE);
@@ -748,21 +741,13 @@ gucharmap_search_dialog_init (GucharmapSearchDialog *search_dialog)
   gtk_dialog_add_button (GTK_DIALOG (search_dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
   priv->prev_button = gtk_button_new ();
-#if GTK_CHECK_VERSION (2,18,0)
   gtk_widget_set_can_default (priv->prev_button, TRUE);
-#else
-  GTK_WIDGET_SET_FLAGS (priv->prev_button, GTK_CAN_DEFAULT);
-#endif
   set_button_stock_image_and_label (GTK_BUTTON (priv->prev_button), GTK_STOCK_GO_BACK, _("_Previous"));
   gtk_dialog_add_action_widget (GTK_DIALOG (search_dialog), priv->prev_button, GUCHARMAP_RESPONSE_PREVIOUS);
   gtk_widget_show (priv->prev_button);
 
   priv->next_button = gtk_button_new ();
-#if GTK_CHECK_VERSION (2,18,0)
   gtk_widget_set_can_default (priv->next_button, TRUE);
-#else
-  GTK_WIDGET_SET_FLAGS (priv->next_button, GTK_CAN_DEFAULT);
-#endif
   gtk_widget_show (priv->next_button);
   set_button_stock_image_and_label (GTK_BUTTON (priv->next_button), GTK_STOCK_GO_FORWARD, _("_Next"));
   gtk_dialog_add_action_widget (GTK_DIALOG (search_dialog), priv->next_button, GUCHARMAP_RESPONSE_NEXT);
@@ -774,19 +759,21 @@ gucharmap_search_dialog_init (GucharmapSearchDialog *search_dialog)
                                            GTK_RESPONSE_CLOSE,
                                            -1);
 
-  hbox = gtk_hbox_new (FALSE, 12);
-  gtk_widget_show (hbox);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-  gtk_box_pack_start (GTK_BOX (content_area), hbox, FALSE, FALSE, 0);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
+  gtk_widget_show (grid);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 6);
+  gtk_box_pack_start (GTK_BOX (content_area), grid, FALSE, FALSE, 0);
 
   label = gtk_label_new_with_mnemonic (_("_Search:"));
   gtk_widget_show (label);
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
 
   priv->entry = gtk_entry_new ();
   gtk_widget_show (priv->entry);
+  gtk_widget_set_hexpand (priv->entry, TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY (priv->entry), TRUE);
-  gtk_box_pack_start (GTK_BOX (hbox), priv->entry, TRUE, TRUE, 0);
+  gtk_grid_attach (GTK_GRID (grid), priv->entry, 1, 0, 1, 1);
   g_signal_connect (priv->entry, "changed", G_CALLBACK (entry_changed), search_dialog);
 
   priv->whole_word_option = gtk_check_button_new_with_mnemonic (_("Match _whole word"));
@@ -866,6 +853,18 @@ gucharmap_search_dialog_present (GucharmapSearchDialog *search_dialog)
   gtk_window_present (GTK_WINDOW (search_dialog));
 }
 
+void
+gucharmap_search_dialog_set_search (GucharmapSearchDialog *search_dialog,
+                                    const char            *search_string)
+{
+  GucharmapSearchDialogPrivate *priv;
+  g_return_if_fail (GUCHARMAP_IS_SEARCH_DIALOG (search_dialog));
+  g_return_if_fail (search_string != NULL);
+
+  priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
+  gtk_entry_set_text (GTK_ENTRY (priv->entry), search_string);
+}
+
 gdouble
 gucharmap_search_dialog_get_completed (GucharmapSearchDialog *search_dialog)
 {
@@ -875,11 +874,7 @@ gucharmap_search_dialog_get_completed (GucharmapSearchDialog *search_dialog)
     return -1.0;
   else
     {
-#if ENABLE_UNIHAN
       gdouble total = gucharmap_get_unicode_data_name_count () + gucharmap_get_unihan_count ();
-#else
-      gdouble total = gucharmap_get_unicode_data_name_count ();
-#endif
       return (gdouble) priv->search_state->strings_checked / total;
     }
 }
